@@ -95,7 +95,7 @@ class plot_tools:
         fig.tight_layout()
         
 
-    def plot_phase_space_ellipse(fig, tracker, axis='both'):
+    def plot_phase_space_ellipse(fig, tracker=None, use_coords_from_tracker=True, x=None, px=None, y=None, py=None, axis='both'):
         """
         Method to plot phase space ellipse from X-suite tracking 
         """
@@ -103,30 +103,40 @@ class plot_tools:
         mpl.rcParams['xtick.labelsize'] = 14
         mpl.rcParams['ytick.labelsize'] = 14
         
+        # Use last recording from tracker, else specified x, px, y, py
+        if use_coords_from_tracker:
+            x = tracker.record_last_track.x
+            px = tracker.record_last_track.px
+            y = tracker.record_last_track.y
+            py = tracker.record_last_track.py
+        else:
+            if [ele for ele in (x, px, y, py) if ele is None]:
+                print("If tracker coordinates are not used, x, px, y, py must be given!")
+                
         fig.suptitle('Phase space ellipse',fontsize=18)
         if axis == 'both':
             ax = fig.add_subplot(2, 1, 1)  # create an axes object in the figure
-            ax.plot(tracker.record_last_track.x, tracker.record_last_track.px, 'ro', markersize=0.2, alpha=0.3)
+            ax.plot(x, px, 'ro', markersize=0.2, alpha=0.3)
             ax.set_ylabel("$p_{x}$")
             ax.set_xlabel("$x$")
             ax2 = fig.add_subplot(2, 1, 2, sharex=ax)  # create a second axes object in the figure
-            ax2.plot(tracker.record_last_track.y, tracker.record_last_track.py, 'bo', markersize=0.2, alpha=0.3)
+            ax2.plot(y, py, 'bo', markersize=0.2, alpha=0.3)
             ax2.set_ylabel("$p_{y}$")
             ax2.set_xlabel("$y$")
         else:
             ax = fig.add_subplot(1, 1, 1)  # create an axes object in the figure
             if axis == 'horizontal':
-                ax.plot(tracker.record_last_track.x, tracker.record_last_track.px, 'ro', markersize=0.2, alpha=0.3)
+                ax.plot(x, px, 'ro', markersize=0.2, alpha=0.3)
                 ax.set_ylabel("$p_{x}$")
                 ax.set_xlabel("$x$")
             if axis == 'vertical':
-                ax.plot(tracker.record_last_track.y, tracker.record_last_track.py, 'bo', markersize=0.2, alpha=0.3)
+                ax.plot(y, py, 'bo', markersize=0.2, alpha=0.3)
                 ax.set_ylabel("$p_{y}$")
                 ax.set_xlabel("$y$")
         fig.tight_layout()
             
             
-    def plot_centroid_motion(fig, tracker, axis='both'):
+    def plot_centroid_motion(fig, tracker=None, use_coords_from_tracker=True, x=None, y=None, axis='both'):
         """
         Method to plot centroid from tracker, either in horizontal or vertical plane 
         """
@@ -136,13 +146,22 @@ class plot_tools:
         
         fig.suptitle('X-suite tracking - centroid motion',fontsize=18)
         
+                
+        # Use last recording from tracker, else specified x, px, y, py
+        if use_coords_from_tracker:
+            x = tracker.record_last_track.x
+            y = tracker.record_last_track.y
+        else:
+            if [ele for ele in (x, y) if ele is None]:
+                print("If tracker coordinates are not used, x, px, y, py must be given!")
+        
         if axis == 'both':
             ax = fig.add_subplot(2, 1, 1)  # create an axes object in the figure
-            ax.plot(np.mean(tracker.record_last_track.x, axis=0), marker='o', color='r', markersize=5)
+            ax.plot(np.mean(x, axis=0), marker='o', color='r', markersize=5)
             ax.set_ylabel("Centroid $X$ [m]")
             ax.set_xlabel("#turns")
             ax2 = fig.add_subplot(2, 1, 2, sharex=ax)  # create a second axes object in the figure
-            ax2.plot(np.mean(tracker.record_last_track.y, axis=0), marker='o', color='b', markersize=5)
+            ax2.plot(np.mean(y, axis=0), marker='o', color='b', markersize=5)
             ax2.set_ylabel("Centroid $Y$ [m]")
             ax2.set_xlabel("#turns")
             plt.setp(ax.get_xticklabels(), visible=False)
@@ -153,11 +172,11 @@ class plot_tools:
             #plt.xticks(fontsize=14)  
             #plt.yticks(fontsize=14)
             if axis == 'horizontal':
-                ax.plot(np.mean(tracker.record_last_track.x, axis=0), marker='o', color='r', markersize=5)
+                ax.plot(np.mean(x, axis=0), marker='o', color='r', markersize=5)
                 ax.set_ylabel("Horizontal centroid $X$ [m]")
                 ax.set_xlabel("#turns")
             if axis == 'vertical':
-                ax.plot(np.mean(tracker.record_last_track.y, axis=0), marker='o', color='b', markersize=5)
+                ax.plot(np.mean(y, axis=0), marker='o', color='b', markersize=5)
                 ax.set_ylabel("Vertical centroid $Y$ [m]")
                 ax.set_xlabel("#turns")
         fig.tight_layout()
